static const struct dis386 evex_table[][256] = {
  /* EVEX_0F */
  {
    /* 00 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 08 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 10 */
    { PREFIX_TABLE (PREFIX_EVEX_0F10) },
    { PREFIX_TABLE (PREFIX_EVEX_0F11) },
    { PREFIX_TABLE (PREFIX_EVEX_0F12) },
    { PREFIX_TABLE (PREFIX_EVEX_0F13) },
    { PREFIX_TABLE (PREFIX_EVEX_0F14) },
    { PREFIX_TABLE (PREFIX_EVEX_0F15) },
    { PREFIX_TABLE (PREFIX_EVEX_0F16) },
    { PREFIX_TABLE (PREFIX_EVEX_0F17) },
    /* 18 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 20 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 28 */
    { PREFIX_TABLE (PREFIX_EVEX_0F28) },
    { PREFIX_TABLE (PREFIX_EVEX_0F29) },
    { PREFIX_TABLE (PREFIX_EVEX_0F2A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F2B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F2C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F2D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F2E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F2F) },
    /* 30 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 38 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 40 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 48 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 50 */
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F51) },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F54) },
    { PREFIX_TABLE (PREFIX_EVEX_0F55) },
    { PREFIX_TABLE (PREFIX_EVEX_0F56) },
    { PREFIX_TABLE (PREFIX_EVEX_0F57) },
    /* 58 */
    { PREFIX_TABLE (PREFIX_EVEX_0F58) },
    { PREFIX_TABLE (PREFIX_EVEX_0F59) },
    { PREFIX_TABLE (PREFIX_EVEX_0F5A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F5B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F5C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F5D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F5E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F5F) },
    /* 60 */
    { PREFIX_TABLE (PREFIX_EVEX_0F60) },
    { PREFIX_TABLE (PREFIX_EVEX_0F61) },
    { PREFIX_TABLE (PREFIX_EVEX_0F62) },
    { PREFIX_TABLE (PREFIX_EVEX_0F63) },
    { PREFIX_TABLE (PREFIX_EVEX_0F64) },
    { PREFIX_TABLE (PREFIX_EVEX_0F65) },
    { PREFIX_TABLE (PREFIX_EVEX_0F66) },
    { PREFIX_TABLE (PREFIX_EVEX_0F67) },
    /* 68 */
    { PREFIX_TABLE (PREFIX_EVEX_0F68) },
    { PREFIX_TABLE (PREFIX_EVEX_0F69) },
    { PREFIX_TABLE (PREFIX_EVEX_0F6A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F6B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F6C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F6D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F6E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F6F) },
    /* 70 */
    { PREFIX_TABLE (PREFIX_EVEX_0F70) },
    { REG_TABLE (REG_EVEX_0F71) },
    { REG_TABLE (REG_EVEX_0F72) },
    { REG_TABLE (REG_EVEX_0F73) },
    { PREFIX_TABLE (PREFIX_EVEX_0F74) },
    { PREFIX_TABLE (PREFIX_EVEX_0F75) },
    { PREFIX_TABLE (PREFIX_EVEX_0F76) },
    { Bad_Opcode },
    /* 78 */
    { PREFIX_TABLE (PREFIX_EVEX_0F78) },
    { PREFIX_TABLE (PREFIX_EVEX_0F79) },
    { PREFIX_TABLE (PREFIX_EVEX_0F7A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F7B) },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F7E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F7F) },
    /* 80 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 88 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 90 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 98 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* A0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* A8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* B0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* B8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* C0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0FC2) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0FC4) },
    { PREFIX_TABLE (PREFIX_EVEX_0FC5) },
    { PREFIX_TABLE (PREFIX_EVEX_0FC6) },
    { Bad_Opcode },
    /* C8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* D0 */
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0FD1) },
    { PREFIX_TABLE (PREFIX_EVEX_0FD2) },
    { PREFIX_TABLE (PREFIX_EVEX_0FD3) },
    { PREFIX_TABLE (PREFIX_EVEX_0FD4) },
    { PREFIX_TABLE (PREFIX_EVEX_0FD5) },
    { PREFIX_TABLE (PREFIX_EVEX_0FD6) },
    { Bad_Opcode },
    /* D8 */
    { PREFIX_TABLE (PREFIX_EVEX_0FD8) },
    { PREFIX_TABLE (PREFIX_EVEX_0FD9) },
    { PREFIX_TABLE (PREFIX_EVEX_0FDA) },
    { PREFIX_TABLE (PREFIX_EVEX_0FDB) },
    { PREFIX_TABLE (PREFIX_EVEX_0FDC) },
    { PREFIX_TABLE (PREFIX_EVEX_0FDD) },
    { PREFIX_TABLE (PREFIX_EVEX_0FDE) },
    { PREFIX_TABLE (PREFIX_EVEX_0FDF) },
    /* E0 */
    { PREFIX_TABLE (PREFIX_EVEX_0FE0) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE1) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE2) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE3) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE4) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE5) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE6) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE7) },
    /* E8 */
    { PREFIX_TABLE (PREFIX_EVEX_0FE8) },
    { PREFIX_TABLE (PREFIX_EVEX_0FE9) },
    { PREFIX_TABLE (PREFIX_EVEX_0FEA) },
    { PREFIX_TABLE (PREFIX_EVEX_0FEB) },
    { PREFIX_TABLE (PREFIX_EVEX_0FEC) },
    { PREFIX_TABLE (PREFIX_EVEX_0FED) },
    { PREFIX_TABLE (PREFIX_EVEX_0FEE) },
    { PREFIX_TABLE (PREFIX_EVEX_0FEF) },
    /* F0 */
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0FF1) },
    { PREFIX_TABLE (PREFIX_EVEX_0FF2) },
    { PREFIX_TABLE (PREFIX_EVEX_0FF3) },
    { PREFIX_TABLE (PREFIX_EVEX_0FF4) },
    { PREFIX_TABLE (PREFIX_EVEX_0FF5) },
    { PREFIX_TABLE (PREFIX_EVEX_0FF6) },
    { Bad_Opcode },
    /* F8 */
    { PREFIX_TABLE (PREFIX_EVEX_0FF8) },
    { PREFIX_TABLE (PREFIX_EVEX_0FF9) },
    { PREFIX_TABLE (PREFIX_EVEX_0FFA) },
    { PREFIX_TABLE (PREFIX_EVEX_0FFB) },
    { PREFIX_TABLE (PREFIX_EVEX_0FFC) },
    { PREFIX_TABLE (PREFIX_EVEX_0FFD) },
    { PREFIX_TABLE (PREFIX_EVEX_0FFE) },
    { Bad_Opcode },
  },
  /* EVEX_0F38 */
  {
    /* 00 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3800) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3804) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 08 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F380B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F380C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F380D) },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 10 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3810) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3811) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3812) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3813) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3814) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3815) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3816) },
    { Bad_Opcode },
    /* 18 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3818) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3819) },
    { PREFIX_TABLE (PREFIX_EVEX_0F381A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F381B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F381C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F381D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F381E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F381F) },
    /* 20 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3820) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3821) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3822) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3823) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3824) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3825) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3826) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3827) },
    /* 28 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3828) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3829) },
    { PREFIX_TABLE (PREFIX_EVEX_0F382A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F382B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F382C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F382D) },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 30 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3830) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3831) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3832) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3833) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3834) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3835) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3836) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3837) },
    /* 38 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3838) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3839) },
    { PREFIX_TABLE (PREFIX_EVEX_0F383A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F383B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F383C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F383D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F383E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F383F) },
    /* 40 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3840) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3842) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3843) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3844) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3845) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3846) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3847) },
    /* 48 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F384C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F384D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F384E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F384F) },
    /* 50 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3850) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3851) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3852) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3853) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3854) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3855) },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 58 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3858) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3859) },
    { PREFIX_TABLE (PREFIX_EVEX_0F385A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F385B) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 60 */
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3862) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3863) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3864) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3865) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3866) },
    { Bad_Opcode },
    /* 68 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3868) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 70 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3870) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3871) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3872) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3873) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3875) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3876) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3877) },
    /* 78 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3878) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3879) },
    { PREFIX_TABLE (PREFIX_EVEX_0F387A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F387B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F387C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F387D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F387E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F387F) },
    /* 80 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3883) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 88 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3888) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3889) },
    { PREFIX_TABLE (PREFIX_EVEX_0F388A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F388B) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F388D) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F388F) },
    /* 90 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3890) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3891) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3892) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3893) },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3896) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3897) },
    /* 98 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3898) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3899) },
    { PREFIX_TABLE (PREFIX_EVEX_0F389A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F389B) },
    { PREFIX_TABLE (PREFIX_EVEX_0F389C) },
    { PREFIX_TABLE (PREFIX_EVEX_0F389D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F389E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F389F) },
    /* A0 */
    { PREFIX_TABLE (PREFIX_EVEX_0F38A0) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38A1) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38A2) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38A3) },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F38A6) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38A7) },
    /* A8 */
    { PREFIX_TABLE (PREFIX_EVEX_0F38A8) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38A9) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38AA) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38AB) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38AC) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38AD) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38AE) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38AF) },
    /* B0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F38B4) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38B5) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38B6) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38B7) },
    /* B8 */
    { PREFIX_TABLE (PREFIX_EVEX_0F38B8) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38B9) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38BA) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38BB) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38BC) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38BD) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38BE) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38BF) },
    /* C0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F38C4) },
    { Bad_Opcode },
    { REG_TABLE (REG_EVEX_0F38C6) },
    { REG_TABLE (REG_EVEX_0F38C7) },
    /* C8 */
    { PREFIX_TABLE (PREFIX_EVEX_0F38C8) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F38CA) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38CB) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38CC) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38CD) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F38CF) },
    /* D0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* D8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F38DC) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38DD) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38DE) },
    { PREFIX_TABLE (PREFIX_EVEX_0F38DF) },
    /* E0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* E8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* F0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* F8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
  },
  /* EVEX_0F3A */
  {
    /* 00 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A00) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A01) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A03) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A04) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A05) },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 08 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A08) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A09) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A0A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A0B) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A0F) },
    /* 10 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A14) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A15) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A16) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A17) },
    /* 18 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A18) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A19) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A1A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A1B) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A1D) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A1E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A1F) },
    /* 20 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A20) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A21) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A22) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A23) },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A25) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A26) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A27) },
    /* 28 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 30 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 38 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A38) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A39) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A3A) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A3B) },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A3E) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A3F) },
    /* 40 */
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A42) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A43) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A44) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 48 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 50 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A50) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A51) },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A54) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A55) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A56) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A57) },
    /* 58 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 60 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A66) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A67) },
    /* 68 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 70 */
    { PREFIX_TABLE (PREFIX_EVEX_0F3A70) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A71) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A72) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3A73) },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 78 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 80 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 88 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 90 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* 98 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* A0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* A8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* B0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* B8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* C0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* C8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { PREFIX_TABLE (PREFIX_EVEX_0F3ACE) },
    { PREFIX_TABLE (PREFIX_EVEX_0F3ACF) },
    /* D0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* D8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* E0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* E8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* F0 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    /* F8 */
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
    { Bad_Opcode },
  },
};
